--- vanessa_socket_server.c	2010-06-22 09:21:37.000000000 +0200
+++ vanessa_socket_server.c	2015-03-09 09:46:09.000000000 +0100
@@ -121,6 +121,7 @@
 				goto err_close;
 			continue;
 		}
+		freeaddrinfo(res);
 		return s;
 	} while ((res = res->ai_next));
 
@@ -374,7 +375,7 @@
 				      const unsigned int maximum_connections,
 				      struct sockaddr *return_from, 
 				      struct sockaddr *return_to,
-				      vanessa_socket_flag_t flag)
+				      vanessa_socket_flag_t flag, long opt)
 {
 	unsigned int addrlen;
 	pid_t child = 0;
@@ -388,11 +389,12 @@
 		addrlen = sizeof(from);
 		*g = accept(listen_socket, (struct sockaddr *) &from, &addrlen);
 		if (*g  < 0) {
-			if(errno == EINTR || errno == ECONNABORTED) {
-				continue; /* Ignore EINTR  and ECONNABORTED */
-			}
-			if (errno == EAGAIN || errno == EWOULDBLOCK)
+			if (opt & O_NONBLOCK &&
+			    (errno == EAGAIN || errno == EWOULDBLOCK))
 				return -1; /* Don't log EAGAIN or EWOULDBLOCK */
+			if (errno == EINTR || errno == ECONNABORTED ||
+			    errno == EAGAIN || errno == EWOULDBLOCK)
+				continue; /* Ignore */
 			VANESSA_LOGGER_DEBUG_ERRNO("accept");
 			return(-1);
 		}
@@ -468,12 +470,19 @@
 {
 	pid_t child;
 	int g;
+	long opt;
+
+	opt = fcntl(listen_socket, F_GETFL, NULL);
+	if (opt < 0) {
+		VANESSA_LOGGER_DEBUG_ERRNO("fcntl: F_GETFL");
+		return -1;
+	}
 
 	while (1) {
 		child = __vanessa_socket_server_accept(&g, listen_socket, NULL,
 		 				       maximum_connections, 
 						       return_from, return_to, 
-						       flag);
+						       flag, opt);
 		if (child < 0) {
 			VANESSA_LOGGER_DEBUG("__vanessa_socket_server_accept");
 			return -1;
@@ -548,14 +557,14 @@
 							listen_socketv,
 							maximum_connections,
 							return_from, return_to,
-							flag);
+							flag, opt);
 	if (child < 0) {
-		if (errno == EAGAIN || errno == EWOULDBLOCK)
-			status = 0;
-		else {
+		status = -1;
+		if (errno == EAGAIN || errno == EWOULDBLOCK) {
+			if (opt & O_NONBLOCK)
+				status = 0;
+		} else
 			VANESSA_LOGGER_DEBUG("__vanessa_socket_server_accept");
-			status = -1;
-		}
 	}
 
 	if (!(opt & O_NONBLOCK) && child &&
@@ -628,6 +637,8 @@
 						maximum_connections, 
 						return_from, return_to, flag);
 			if (child < 0) {
+				if (errno == EAGAIN || errno == EWOULDBLOCK)
+					continue;
 				VANESSA_LOGGER_DEBUG(
 					"__vanessa_socket_server_acceptv");
 				goto err;

